﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.CloudManager.Model;
using HeuristicLab.Core;

namespace HeuristicLab.Clients.Hive.CloudManager.Views {
  public partial class AddAzureServiceDialog : Form {
    private BindingList<Subscription> subscriptions;
    private BindingList<HostedService> hostedServices;
    private BindingList<string> locations;
    private BindingList<AffinityGroup> affinityGroups;

    private BackgroundWorker worker;

    private bool isInitializing;
    private bool bwCompleted;
    private bool closePending;

    public AddAzureServiceDialog(IItemList<Subscription> subs) {
      InitializeComponent();
      isInitializing = true;

      this.subscriptions = new BindingList<Subscription>(subs);
      this.hostedServices = new BindingList<HostedService>();
      this.locations = new BindingList<string>();
      this.affinityGroups = new BindingList<AffinityGroup>();

      SetLookupBinding(cmbLocation, locations, "", "");
      SetLookupBinding(cmbAffinityGroup, affinityGroups, "Label", "Name");
      SetLookupBinding(cmbChooseHostedService, hostedServices, "ServiceName", "ServiceName");
      SetLookupBinding(cmbChooseSubscription, subscriptions, "SubscriptionName", "SubscriptionID");

      if (!cbNewHostedService.Checked) {
        SetGroupBoxControlsEnabled(gbNewHostedService, cbNewHostedService.Checked);
      }

      worker = new BackgroundWorker();
      worker.DoWork += new DoWorkEventHandler(UpdateTask);
      worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(WorkerCompleted);


      isInitializing = false;
      cmbChooseSubscription_SelectedValueChanged(this, EventArgs.Empty);
    }

    protected override void OnFormClosing(FormClosingEventArgs e) {
      if (!bwCompleted) {
        worker.CancelAsync();
        this.Enabled = false;
        e.Cancel = true;
        closePending = true;
      }
    }

    private void SetLookupBinding(ListControl toBind, object dataSource, string displayMember, string valueMember) {
      toBind.DisplayMember = displayMember;
      toBind.ValueMember = valueMember;
      toBind.DataSource = dataSource;
    }

    private void SetGroupBoxControlsEnabled(GroupBox gb, bool isEnabled) {
      foreach (Control ctrl in gb.Controls) {
        ctrl.Enabled = isEnabled;
      }
    }

    private void SetAllControlsEnabled(Form frm, bool isEnabled) {
      foreach (Control ctrl in frm.Controls) {
        if (!(ctrl is ProgressBar)) {
          this.Invoke((MethodInvoker)delegate { ctrl.Enabled = isEnabled; });
        }
      }
    }

    private void SetBindingList<T>(ref BindingList<T> bindinglist, List<T> list) {
      if (list != null) {
        BindingList<T> newList = new BindingList<T>(list);
        bindinglist = newList;
      }
    }

    private void UpdateBindingList<T>(BindingList<T> bindinglist, List<T> list) {
      if (list != null) {
        bindinglist.Clear();
        foreach (T item in list)
          bindinglist.Add(item);
      }
    }

    private void cbNewHostedService_CheckedChanged(object sender, EventArgs e) {
      if (cbNewHostedService.Checked) {
        Subscription subscription = (Subscription)cmbChooseSubscription.SelectedItem;
        if (subscription.CurrentHostedServices == subscription.MaxHostedServices) {
          MessageBox.Show("You have already created the maximum number of hosted services", "Limit Reached", MessageBoxButtons.OK, MessageBoxIcon.Error);
          cbNewHostedService.Checked = false;
        } else {
          SetGroupBoxControlsEnabled(gbNewHostedService, cbNewHostedService.Checked);
          cmbChooseHostedService.Enabled = false;
        }
      } else {
        // checked == false
        SetGroupBoxControlsEnabled(gbNewHostedService, false);
        if (hostedServices == null) {
          cbNewHostedService.Checked = true;
        }
      }
    }

    private void cmbChooseSubscription_SelectedValueChanged(object sender, EventArgs e) {
      if (!isInitializing) {
        Subscription item = (Subscription)cmbChooseSubscription.SelectedItem;
        if (item != null) {
          progressBar.Visible = true;
          worker.RunWorkerAsync(item);
        }
      }
    }

    private void btnAddCertificate_Click(object sender, EventArgs e) {
      using (var form = new AddCertificate()) {
        form.ShowDialog();
        Debug.WriteLine("End - AddCertificateDialog");
      }
    }

    private void btnCancel_Click(object sender, EventArgs e) {
      this.Close();
    }

    private void btnOk_Click(object sender, EventArgs e) {
      if (cmbChooseSubscription.SelectedItem != null) {
        if (cbNewHostedService.Checked) {
          if (rbRegion.Checked) {
            string serviceName = tbServiceName.Text;
            string label = tbLabel.Text;
            string location = (string)cmbLocation.SelectedItem;
            Subscription sub = (Subscription)cmbChooseSubscription.SelectedItem;
            CloudManagerClient.Instance.AzureProvider.CreateHostedService(sub, serviceName, label, "Hosted service for hive slave", location);
          } else {
            // rbAffinitGroup.Checked
          }
        }

      }
    }

    private void UpdateTask(object sender, DoWorkEventArgs e) {
      try {
        SetAllControlsEnabled(this, false);

        Subscription item = (Subscription)e.Argument;
        item = CloudManagerClient.Instance.AzureProvider.GetSubscriptionInfo(item.SubscriptionID, item.CertificateThumbprint);
        this.Invoke((MethodInvoker)delegate { lblCores.Text = item.CurrentCoreCount + " / " + item.MaxCoreCount; });

        List<HostedService> services = CloudManagerClient.Instance.AzureProvider.ListHostedServices(item);
        List<string> locs = CloudManagerClient.Instance.AzureProvider.ListLocations(item);
        List<AffinityGroup> groups = CloudManagerClient.Instance.AzureProvider.ListAffinityGroups(item);

        this.Invoke((MethodInvoker)delegate { UpdateBindingList<HostedService>(hostedServices, services); });
        this.Invoke((MethodInvoker)delegate { UpdateBindingList<string>(locations, locs); });
        this.Invoke((MethodInvoker)delegate { UpdateBindingList<AffinityGroup>(affinityGroups, groups); });

        SetAllControlsEnabled(this, true);
      }
      catch (Exception ex) {
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void WorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
      this.Invoke((MethodInvoker)delegate { progressBar.Visible = false; });
      bwCompleted = true;
      if (closePending) {
        this.Close();
      }
    }
  }
}
